package de.hdm_stuttgart.sw1.klausur.test;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.bitte_ignorieren.Marking;
import de.hdm_stuttgart.mi.bitte_ignorieren.MarkingTest;
import static de.hdm_stuttgart.sw1.klausur.StringTransforms.chinesischerAkzent;
import static de.hdm_stuttgart.sw1.klausur.StringTransforms.entferneVokale;
import static de.hdm_stuttgart.sw1.klausur.StringTransforms.morsen;

@SuppressWarnings({"static-method", "javadoc"})
public class Test_StringTransforms extends MarkingTest {


	// Tests zu chinesischerAkzent(final String input)
	//
	@Test
	public void chinesischerAkzent_EmptyString () {
		Assert.assertEquals("", chinesischerAkzent(""));
	}

	@Test
	@Marking(points=2)
	public void chinesischerAkzent_Identisch() {
		final String ohne_r_buchstaben = "Es geht auch ohne!";
		Assert.assertEquals(ohne_r_buchstaben, chinesischerAkzent(ohne_r_buchstaben));
	}

	@Test
	@Marking(points=2)
	public void chinesischerAkzent_Multiple() {
		Assert.assertEquals("Almes Lumänien", chinesischerAkzent("Armes Rumänien"));
	}

	// Tests zu entferneVokale(final String input)
	//
	@Test
	@Marking(points=4)
	public void entferneVokale_NullInput () {
		Assert.assertNull(entferneVokale(null));
	}

	@Test
	@Marking(points=2)
	public void entferneVokale_EmptyString() {
		Assert.assertEquals("", entferneVokale(""));
	}

	@Test
	public void entferneVokale_Einzelvokal() {
		Assert.assertEquals("", entferneVokale("E"));
	}

	@Test
	public void entferneVokale_Einzelkonsonant() {
		Assert.assertEquals("k", entferneVokale("k"));
	}

	@Test
	@Marking(points=2)
	public void entferneVokale_NixZuTun () {
		final String textOhneVokale = "dffGl1883KQW_+";
		Assert.assertEquals(textOhneVokale, entferneVokale(textOhneVokale));
	}

	@Test
	@Marking(points=5)
	public void entferneVokale_Multiple(){
		Assert.assertEquals("Bssr chtng f lls!", entferneVokale("Besser Achtung auf alles!"));
	}

	// Tests zu morsen(final String input)
	//
	@Test
	public void morsen_Leer () {
		Assert.assertEquals("", morsen(""));
	}

	@Test
	@Marking(points=2)
	public void morsen_Abend () {
		Assert.assertEquals(".- -... . -. -.. ", morsen("Abend"));
	}
}
