package de.hdm_stuttgart.sw1.klausur.test;

import static de.hdm_stuttgart.sw1.klausur.Spielen.getTeilnehmerNaechsteRunde;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.Timeout;

import de.hdm_stuttgart.mi.exam.unitmarking.Marking;

@SuppressWarnings({"javadoc"})
public class Test_SpielNaechsteRunde  {

   @Rule // Kill test if lasting longer than 1000 milliseconds
   public Timeout globalTimeout = new Timeout(1000);

	@Test
	public void nullBehandlung () {
		Assert.assertEquals(
				null, 
				getTeilnehmerNaechsteRunde(null));
	}

	@Test
	public void einzelnerTeilnehmer () {
		final int[] punkte= {44};
		Assert.assertArrayEquals(
				punkte, 
				getTeilnehmerNaechsteRunde(punkte));
	}
	
	@Test
	public void zweiSpielerGleich () {
		final int[] punkte= {2, 2};
		Assert.assertArrayEquals(
				punkte, 
				getTeilnehmerNaechsteRunde(punkte));
	}
	
	@Test
	public void zweiSpielerUngleich () {
		final int[] punkte= {2, 1};
		Assert.assertArrayEquals(
				new int[]{2}, 
				getTeilnehmerNaechsteRunde(punkte));
	}
	
	@Test
	public void identischerPunktstand () {
		final int[] punkte= {2, 2, 2};
		Assert.assertArrayEquals(
				punkte, 
				getTeilnehmerNaechsteRunde(punkte));
	}
	
	@Test
	public void dreiSpielerZweiGewinnerLinks () {
		final int[] 
				aktuell = {3, 3, 2},
				naechste= {3, 3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	public void dreiSpielerZweiGewinnerRechts () {
		final int[] 
				aktuell = {2, 3, 3},
				naechste= {3, 3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	public void dreiSpielerLinks () {
		final int[] 
				aktuell = {3, 2, 2},
				naechste= {3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	public void dreiSpielerMitte () {
		final int[] 
				aktuell = {2, 3, 2},
				naechste= {3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	public void dreiSpielerRechts () {
		final int[] 
				aktuell = {2, 2, 3},
				naechste= {3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	public void vierSpieler () {
		final int[] 
				aktuell = {1,2,3,2},
				naechste= {3} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
	
	@Test
	@Marking(points=2)
	public void siebenSpielerDreiGewinner () {
		final int[] 
				aktuell = {5, 1, 1, 2, 1, 1, 5},
				naechste= {5, 2, 5} ;
		Assert.assertArrayEquals(naechste, getTeilnehmerNaechsteRunde(aktuell));
	}
}