package de.hdm_stuttgart.mi.sda1.exam.test.ignore;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.Document;
import org.jdom2.JDOMException;
import org.jdom2.ProcessingInstruction;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/**
 * Evaluate a given set of XML document instances
 *
 */
public class InstanceSetEvaluation {
   
   /**
    * Base directory containing xml test file instances.
    */
   public final String xmlTestFileDir;
   
   /**
    * true if all tests have been successful.
    */
   public final boolean allTestsSucceeded;
   
   private static Logger log = LogManager.getLogger(InstanceSetEvaluation.class);
   
   final static List<InstanceTest> tests = new Vector<>();
   
   final StringBuffer messages = new StringBuffer(), errorMessages = new StringBuffer();

   /**
    * @param args unused
    */
   public static void main(String[] args) {
      final InstanceSetEvaluation ise = new InstanceSetEvaluation("SchemaTest");
      System.out.println(ise.getMessages());
      System.out.println("All tests succeeded:" + ise.allTestsSucceeded);
   }

   /**
    * @return Individual failed test(s) error message(s).
    */
   public String getErrorMessages() {
      return errorMessages.toString();
   }
   
   /**
    * @return Individual test's and aggregated results.
    */
   public String getMessages() {
      return messages.toString();
   }
   
   /**
    * 
    * @param xmlTestFileDir Directory containing XML instances to be processed.
    */
   public InstanceSetEvaluation(final String xmlTestFileDir) {
      this.xmlTestFileDir = xmlTestFileDir;
	   	   
	  final File rootDirectory = new File(xmlTestFileDir);
	  
	  for (final File f: rootDirectory.listFiles(
			  path -> path.getPath().endsWith(".xml"))) {
		  readTestHeader(f);
	  }
    
     tests.forEach(t -> messages.append(t + "\n"));      
     tests.stream().filter(t -> !t.testSucceeded).forEach(t -> errorMessages.append(t + "\n"));      

     allTestsSucceeded = tests.
           stream().
           map(t -> t.testSucceeded).
           reduce((a, b) -> a && b).
           get(); 

	  int reachedPoints = 0, maxPoints = 0;
	  
	  for (final InstanceTest t: tests) {
	     maxPoints += t.reachablePoints;
        if (t.testSucceeded) {
           reachedPoints += t.reachablePoints;
        }
	  }
     messages.append(reachedPoints + " of " + maxPoints +
           " points have been reached");
   }

    void readTestHeader(final File instanceFilename) {
      
      final SAXBuilder metainfoParser = new SAXBuilder(XMLReaders.NONVALIDATING);
      try {
         final Document doc = metainfoParser.build(instanceFilename);
         
         final XPathExpression<ProcessingInstruction> searchHeader = 
               XPathFactory.instance().compile(
                     "/processing-instruction('xmlTest')", 
               Filters.processinginstruction());
         final List<ProcessingInstruction> xmlTestList = 
               searchHeader.evaluate(doc);
         
         switch (xmlTestList.size()) {
         case 0:
            log.info("No 'xmlTest PI found, possible dependency file");
            break;
         case 1:
            tests.add(new InstanceTest(xmlTestList.get(0), xmlTestFileDir, instanceFilename));
            break;
            default:
               log.error("Fatal: Found " + xmlTestList.size() +
                     " <?xmltest ... ?> annotations in file '" + 
                     instanceFilename + "'");
               System.exit(1);
         }        
         
      } catch (final JDOMException e) {
         log.fatal("Document '" + instanceFilename.getPath() +
               "' is invalid:", e);
         System.exit(1);
      } catch (final IOException e) {
         log.fatal("Unable to open '" + instanceFilename.getPath() + 
               "' for reading:", e);
         System.exit(1);
      }
   }
}
